﻿// ****************************************************************************
// *                                                                          *
// * © Copyright 2020 RapidApps Sweden                                        *
// *                                                                          *
// * File       : Program.cs                                                  *
// * Created    : 2020-05-08                                                  *
// * Created by : metscore                                                    *
// *                                                                          *
// ****************************************************************************

using System;
using System.Threading;

namespace PassMarkUSBTestPlug
{
    class Program
    {
        enum RunState { Undefined, RunForPredefinedTime, RunForPredefinedAmount, ShowUSBVoltage, ShowDeviceInfo, Cancel };

        static void Main(string[] args)
        {
            RunState runState = RunState.Undefined;
            while (runState == RunState.Undefined)
            {
                Console.WriteLine("");
                Console.WriteLine("Press one of the following keys:");
                Console.WriteLine("T - Run the program for a predefined time");
                Console.WriteLine("M - Run the program for a predefined amount of data");
                Console.WriteLine("V - Display USB voltage");
                Console.WriteLine("D - Display device info");
                Console.WriteLine("Q - Quit the program");

                while (runState == RunState.Undefined)
                {
                    if (Console.KeyAvailable)
                    {
                        var keyPressed = Console.ReadKey(true);
                        switch (keyPressed.Key)
                        {
                            case ConsoleKey.T:
                                runState = RunState.RunForPredefinedTime;
                                break;
                            case ConsoleKey.M:
                                runState = RunState.RunForPredefinedAmount;
                                break;
                            case ConsoleKey.Q:
                                runState = RunState.Cancel;
                                break;
                            case ConsoleKey.V:
                                runState = RunState.ShowUSBVoltage;
                                break;
                            case ConsoleKey.D:
                                runState = RunState.ShowDeviceInfo;
                                break;
                        }
                    }
                    else
                    {
                        Thread.Sleep(200);
                    }
                }

                if (runState != RunState.Cancel)
                {

                    Console.Clear();
                    USBLoopBackTestFunctions usbTest = new USBLoopBackTestFunctions("ID1", InfoMessage, ErrorMessage);
                    usbTest.AutomaticallyAddNewLine = true;
                    try
                    {
                        usbTest.ShowProgressTick = true;
                        usbTest.Initialize();
                        if (runState == RunState.RunForPredefinedTime) usbTest.ExecuteLoopbackTestForSpecificTime(10);
                        if (runState == RunState.RunForPredefinedAmount) usbTest.ExecuteLoopbackTestForSpecificNumberOfMegaBytes(100);
                        if (runState == RunState.ShowUSBVoltage) usbTest.ShowCurrentUSBVoltage();
                        if (runState == RunState.ShowDeviceInfo) usbTest.ShowConnectedDeviceInfo();
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine($"Error : {ex.Message}");
                    }
                    finally
                    {
                        runState = RunState.Undefined;
                    }
                }
            }
        }

        public static void InfoMessage(object tag, string message)
        {
            Console.Write(message);
        }

        public static void ErrorMessage(object tag, string message, Exception ex)
        {
            Console.WriteLine($"{message}.{(ex is null ? "" : $" [Exception : {ex.Message}]")}");
        }
    }
}
